/****************************************************************************
 * Projekt - Messtechnik 1
 * (c) 2017 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Aufbau und Nutzung der Verbindung zum MQTT Server
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <Ethernet2.h>
#include <PubSubClient.h>
#include "globals.h"
#include "debug.h"
#include "mqttclient.h"

// Variablen -----------------------------------------------------------------
static byte mac[] = {0x90, 0xA2, 0xDA, 0x10, 0x4f, 0x44}; // Ethernet MAC Adresse
static IPAddress    pscsrv(192, 168, 198, 20);            // IP Adresse des MQTT Servers
static unsigned int pscport = 1883;                       // MQTT Port
EthernetClient Ether;
PubSubClient pscclient(Ether);

// Funktionen ---------------------------------------------------------------
boolean MQTTSetup(void)
{
  if (Ethernet.begin(mac) == 0) // Fehlerfall
  {
    return(false);
  }
  else
  {
    pscclient.setServer(pscsrv, pscport);
    return(true);
  }
}

byte MQTTGetIPAddress(byte bno)
{
  return(Ethernet.localIP()[bno]);
}

void MQTTConnectAndPublish(void)
{
  char tmpstr[4];
  char stamp[17];
  
  if (pscclient.connect("JGE-TL")) 
  {
    DebugPrint("MQTT verbunden");
    // Den Zeitstempel erzeugen
    sprintf(stamp, "%02i.%02i.20%02i %02i:%02i", Get_Tag(), Get_Monat(), Get_Jahr(), Get_Stunde(), Get_Minute());
    
    // Nach dem connect folgen die Topics
    sprintf(tmpstr,"%02i",Get_Luftfeuchte());            // Messwert umwandeln in ASCII String
    pscclient.publish("Wohnung/Raum1/Luftfeuchte/Messwert",tmpstr);
    pscclient.publish("Wohnung/Raum1/Luftfeuchte/Messzeit",stamp);
#ifdef DEBUG
    Serial.print("Luftfeuchte Messwert: ");
    Serial.println(tmpstr);
    Serial.print("Luftfeuchte Messzeit: ");
    Serial.println(stamp);
#endif
    sprintf(tmpstr,"%02i",Get_Temperatur());             // Messwert umwandeln in ASCII String
    pscclient.publish("Wohnung/Raum1/Temperatur/Messwert",tmpstr);
    pscclient.publish("Wohnung/Raum1/Temperatur/Messzeit",stamp);
#ifdef DEBUG
    Serial.print("Temperatur Messwert: ");
    Serial.println(tmpstr);
    Serial.print("Temperatur Messzeit: ");
    Serial.println(stamp);
#endif
  } 
  else 
  {
#ifdef DEBUG
    Serial.print("MQTT failed, status = ");
    Serial.print(pscclient.state());
    Serial.println();
#endif
  }
}


