/****************************************************************************
 * Projekt - Messtechnik 1
 * (c) 2017 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Einlesen der Netzwerkzeit ueber Ethernet
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <arduino.h>
#include <Ethernet2.h>
#include "globals.h"
#include "debug.h"
#include "ntpclient.h"
#include "sommerzeit.h"

// Defines -----------------------------------------------------------------
#define SEKUNDEN_PRO_TAG    86400L
#define SEKUNDEN_PRO_STUNDE 3600
#define SEKUNDEN_PRO_MINUTE 60
#define SEVENTY_YEARS       2208988800UL

// Variablen -----------------------------------------------------------------
static IPAddress ip(192, 168, 178, 1);      // IP Adresse NTP Server
static unsigned int localPort    = 8888;    // Port für UDP pakete
static const int NTP_PACKET_SIZE = 48;      // NTP Zeit information
static byte packetBuffer[NTP_PACKET_SIZE];  // NTP paketpuffer
EthernetUDP Udp;

// lokale Prototypen --------------------------------------------------------
static void NTPGetTime(void);
static unsigned long NTPsendpacket(IPAddress address);

// Funktionen ---------------------------------------------------------------
void NTP_Get(void)
{
  Udp.begin(localPort);
  NTPsendpacket(ip);                           // Sende NTP paket zum Zeit Server
  delay(1000);                                 // Warte auf Antwort
  DebugPrint("- NTP Abfrage -");
  NTPGetTime();                                // Antwort einlesen
}

static void NTPGetTime(void)                                                   // aus der Beispielsoftware entnommen und angepasst
{
  unsigned long highWord      = 0ul;
  unsigned long lowWord       = 0ul;
  unsigned long secsSince1900 = 0ul;
  unsigned long epoch         = 0ul;
  
  if ( Udp.parsePacket() ) 
  {
    Udp.read(packetBuffer, NTP_PACKET_SIZE);                            // Paket empfangen und einlesen
    highWord      = word(packetBuffer[40], packetBuffer[41]);
    lowWord       = word(packetBuffer[42], packetBuffer[43]);
    secsSince1900 = highWord << 16 | lowWord;                           // NTP Zeit
    epoch         = secsSince1900 - SEVENTY_YEARS;                      // Unix Zeit beginnt am 1. Jan 1970
    Set_Stunde(((epoch % SEKUNDEN_PRO_TAG) / SEKUNDEN_PRO_STUNDE) + 1); // hiesige Zeitzone ist UTC + 1
    Set_Minute( (epoch  % SEKUNDEN_PRO_STUNDE) / SEKUNDEN_PRO_MINUTE);
    DebugPrintLongValue("Sekunden seit dem 1. Januar 1900 = ",secsSince1900);
    DebugPrintLongValue("Unix Zeit = ",epoch);
#ifdef DEBUG    
    Serial.print("UTC + 1 Zeit ist ");                                  // UTC ist die Zeit am Greenwich Meridian (GMT)
    Serial.print(Get_Stunde());                                         // Ausgabe Stunde
    Serial.print(':');
    if ( ((epoch % SEKUNDEN_PRO_STUNDE) / SEKUNDEN_PRO_MINUTE) < 10 ) 
    { 
      Serial.print('0');                                                // In den ersten 10 minuten eine führende '0' ausgeben
    }
    Serial.println(Get_Minute());                                       // Ausgabe Minuten 
#endif   
    if (CheckSommerzeit() == SommerZeit)
    {
      DebugPrint("Sommerzeit");   
      Set_Stunde(Get_Stunde() + (byte)1);
    }
    else
    {
      DebugPrint("Winterzeit");
    }
  }
}
  
static unsigned long NTPsendpacket(IPAddress address) // NTP Anfrage senden
{
  memset(packetBuffer, 0, NTP_PACKET_SIZE); // set all bytes in the buffer to 0
  // Initwerte fuer den NTP Request
  packetBuffer[0]  = 0b11100011;            // LI, Version, Mode
  packetBuffer[1]  = 0;                     // Stratum, or type of clock
  packetBuffer[2]  = 6;                     // Polling Interval
  packetBuffer[3]  = 0xEC;                  // Peer Clock Precision
  // bytes 4 bis 11: 8 bytes mit '0' fuer Root Delay & Root Dispersion
  packetBuffer[12] = 49;
  packetBuffer[13] = 0x4E;
  packetBuffer[14] = 49;
  packetBuffer[15] = 52;

  Udp.beginPacket(address, 123);            //NTP Abfragen auf Port 123
  Udp.write(packetBuffer, NTP_PACKET_SIZE);
  Udp.endPacket();
}

