/****************************************************************************
 * Projekt - Messtechnik 1
 * (c) 2017 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Konfigurieren und Lesen / Schreiben der Uhrzeit fuer das DS1302 RTC Modul
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <arduino.h>
#include <TimeLib.h>
#include <DS1302RTC.h>
#include "globals.h"
#include "rtclock.h"
#include "debug.h"

// Defines ------------------------------------------------------------------
// RTC Pins
// DS1302:  CE pin    -> Arduino Digital 27
//          I/O pin   -> Arduino Digital 29
//          SCLK pin  -> Arduino Digital 31
//          GND pin   -> Arduino Digital 33
//          VCC pin   -> Arduino Digital 35
#define DS1302_GND_PIN 33
#define DS1302_VCC_PIN 35

// Variablen ----------------------------------------------------------------
DS1302RTC RTC(27, 29, 31);                       // Set pins:  CE, IO,CLK

// Funktionen ---------------------------------------------------------------
void RTCActivate(void)
{
  digitalWrite(DS1302_GND_PIN, LOW);             // GND = 0 Volt
  pinMode(DS1302_GND_PIN, OUTPUT);
  digitalWrite(DS1302_VCC_PIN, HIGH);            // VCC = +5 Volt
  pinMode(DS1302_VCC_PIN, OUTPUT);
  DebugPrint("- RTC aktiviert -");  
  delay(500);                                    // Warten auf Chip...
}

void RTCCheckStatus(void)
{
  if (RTC.haltRTC())                             // Check RTC Clock Oscillator 
  {
    DebugPrint("RTC Clock steht");
  }
  else
    DebugPrint("RTC Clock laeuft");
  if (RTC.writeEN())                             // Check RTC Schreibschutz
    DebugPrint("RTC RAM Schreiben erlaubt");
  else
    DebugPrint("RTC RAM Schreibgeschuetzt");
  delay(2000);                                   // Warten auf Chip...
}

void RTCStart(void)
{
  RTC.haltRTC((uint8_t)0);
  delay(2000);
}

void RTCStop(void)
{
  RTC.haltRTC((uint8_t)1);
  delay(2000);
}

void RTCSetWriteEnable(void)
{
  RTC.writeEN((uint8_t)0);
}

void RTCClearWriteEnable(void)
{
  RTC.writeEN((uint8_t)1);
}

void RTCRead(void)
{
  tmElements_t tm;
  uint8_t ret = (uint8_t)0;

  ret = RTC.read(tm);
  delay(2000);          // Zeit zum Einlesen
  if ((uint8_t)0 == ret)
  {
    DebugPrint("RTC eingelesen");
  }
  else
  {
    DebugPrint("RTC Fehler beim Einlesen");
  }
  Set_Tag((byte)tm.Day);
  Set_Monat((byte)tm.Month);
  // tmYearToY2k() wird hier benutzt um die Bibliotheksfunktion zu ueberlisten
  // Die Bibliotheksfunktion ist fuer die Verwendung unter Linux geschrieben
  // Ein Problem existiert fuer diese Library mit Jahren ueber 69, da intern nur von 0 - 99 funktioniert
  Set_Jahr((byte)tmYearToY2k(tm.Year));
  Set_Stunde((byte)tm.Hour);
  Set_Minute((byte)tm.Minute); 
  Set_Wochentag((byte)tm.Wday);
  DebugPrintDatumZeit(Get_Tag(),Get_Monat(),Get_Jahr(),Get_Stunde(),Get_Minute(),Get_Wochentag());
}

void RTCWrite(void)
{
  tmElements_t tm;
  time_t tmp;
  uint8_t ret = (uint8_t)0;

  tm.Second = (uint8_t)0;
  tm.Minute = Get_Minute();
  tm.Hour   = Get_Stunde();
  tm.Day    = Get_Tag();
  tm.Month  = Get_Monat();
  // Y2kYearToTM() wird hier benutzt um die Bibliotheksfunktion zu ueberlisten
  // Die Bibliotheksfunktion ist fuer die Verwendung unter Linux geschrieben
  // Ein Problem existiert fuer diese Library mit Jahren ueber 69, da intern nur von 0 - 99 funktioniert
  tm.Year   = y2kYearToTm(Get_Jahr());
  tm.Wday   = Get_Wochentag();
  ret = RTC.write(tm);
  delay(2000);
  if((uint8_t)255 == ret)
  {
    DebugPrint("RTC Fehler (WP not disabled)");
  }
  else
  {
    DebugPrint("RTC geschrieben!");
  }
}

