/****************************************************************************
 * Projekt - Messtechnik 1
 * (c) 2017 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog
 * Funktionalitaet:
 * Konfiguration und Einlesen der Messwerte fuer das DHT11 Messmodul
 ***************************************************************************/
// Includes -----------------------------------------------------------------
// Temp/Feuchte Sensor
#include <Adafruit_Sensor.h>
#include <DHT_U.h>
#include <DHT.h>
#include "globals.h"
#include "sensordht11.h"
#include "debug.h"

// Defines -----------------------------------------------------------------
// Sensor Pins
// DHT11:   GND pin   -> Arduino Digital 45
//          VCC pin   -> Arduino Digital 47
//          I/O pin   -> Arduino Digital 49
#define DHT11_GND_PIN 45
#define DHT11_VCC_PIN 47
#define DHTPIN        49 // DHT11 data pin
#define DHTTYPE    DHT11

// Variablen ----------------------------------------------------------------
sensor_t sensor;           // Sensortyp
sensors_event_t event;     // Sensor Messwerte
uint32_t delayMS;          // Sensor Wartezeit
DHT_Unified dht(DHTPIN, DHTTYPE);

// Funktionen ---------------------------------------------------------------
void DHT11Start(void)
{
  dht.begin();                                   // Schnittstelle init  
}

void DHT11Wait(void)
{
  delayMS = sensor.min_delay / 1000;             // Wartezeit berechnen
  DebugPrintLongValueUnit("Verzoegerungszeit: ",delayMS," ms");
}
void DHT11Activate(void)
{
  digitalWrite(DHT11_GND_PIN, LOW);              // GND = 0 Volt
  pinMode(DHT11_GND_PIN, OUTPUT);
  digitalWrite(DHT11_VCC_PIN, HIGH);             // VCC = +5 Volt
  pinMode(DHT11_VCC_PIN, OUTPUT);
  DebugPrint("- DHT aktiviert -");  
  delay(1000);                                   // Warten auf Chip...  
}

void DHT11GetInfoTemperature(void)
{
  dht.temperature().getSensor(&sensor);
  DebugPrint("--- Temperatur ---");
  DebugPrint2("Sensor:       ",sensor.name);
  DebugPrintValue("Driver Ver:   ",sensor.version);
  DebugPrintLongHexValue("Unique ID:    ",sensor.sensor_id);
  DebugPrintValueUnit("Max Value:    ",sensor.max_value," *C");
  DebugPrintValueUnit("Min Value:    ",sensor.min_value," *C");
  DebugPrintValueUnit("Resolution:   ",sensor.resolution," *C");
}

void DHT11GetInfoHumidity(void)
{
  dht.humidity().getSensor(&sensor);
  DebugPrint("--- relative Luftfeuchte ---");
  DebugPrint2("Sensor:       ",sensor.name);
  DebugPrintValue("Driver Ver:   ",sensor.version);
  DebugPrintLongHexValue("Unique ID:    ",sensor.sensor_id);
  DebugPrintValueUnit("Max Value:    ",sensor.max_value," %");
  DebugPrintValueUnit("Min Value:    ",sensor.min_value," %");
  DebugPrintValueUnit("Resolution:   ",sensor.resolution," %");
}

byte DHT11ReadTemperature(void)
{
  byte retval = (byte)0;
  
  delay(delayMS * 2);                       // Wartezeit einhalten
  dht.temperature().getEvent(&event);       // Temperatur einlesen
  if (isnan(event.temperature)) 
  {
    DebugPrint("DHT11 Temperatur Lesefehler!");
  }
  else 
  {
    DebugPrintValueUnit("aktuelle Temperatur:  ",event.temperature," *C");
    retval = byte((event.temperature + (float)0.5));
  }
  return(retval);
}

byte DHT11ReadHumidity(void)
{
  byte retval = (byte)0;

  delay(delayMS * 2);                       // Wartezeit einhalten
  dht.humidity().getEvent(&event);          // Luftfeuchte einlesen
  if (isnan(event.relative_humidity)) 
  {
    DebugPrint("DHT11 Luftfeuchte Lesefehler!");
  }
  else 
  {
    DebugPrintValueUnit("relative Luftfeuchte: ",event.relative_humidity," %");
    retval = byte((event.relative_humidity + (float)0.5));
  }
  return(retval);
}

