/****************************************************************************
 * Sommerzeit erkennen
 * (c) 2017 softwareentwicklung-als-prozess.de
 * Dokumentation s. Blog 
 * Funktionalitaet:
 * Berechnen von Anfang und Ende der Sommerzeit
 ***************************************************************************/
// Includes -----------------------------------------------------------------
#include <arduino.h>
#include "globals.h"
#include "sommerzeit.h"

// Defines -----------------------------------------------------------------
#define SONNTAG    (byte)1
#define MONTAG     (byte)2
#define DIENSTAG   (byte)3
#define MITTWOCH   (byte)4
#define DONNERSTAG (byte)5
#define FREITAG    (byte)6
#define SAMSTAG    (byte)7

#define MAERZ      (byte)3
#define APRIL      (byte)4
#define MAI        (byte)5
#define JUNI       (byte)6
#define JULI       (byte)7
#define AUGUST     (byte)8
#define SEPTEMBER  (byte)9
#define OKTOBER    (byte)10

// lokale Prototypen --------------------------------------------------------
static JahresZeiten OberhalbDiagonale(void);
static JahresZeiten UnterhalbDiagonale(void);

// Funktionen ---------------------------------------------------------------
JahresZeiten CheckSommerzeit(void)
{
  JahresZeiten ret = WinterZeit;
  // vom 1.4. bis 24.10. ist sicher Sommerzeit
  switch(Get_Monat())
  {
    case APRIL:
    case MAI:
    case JUNI:
    case JULI:
    case AUGUST:
    case SEPTEMBER:
      ret = SommerZeit;
      break;
    case OKTOBER:
      ret = SommerZeit; /* bis zum 24.! */
      if ((byte)25 > Get_Tag())
      {
        ret = WinterZeit;
      }
      else
      {
        ret = SommerZeit;
      }
  }

  // Die letzte Woche von Maerz und Oktober muss untersucht werden
  if ((MAERZ == Get_Monat()) || (OKTOBER == Get_Monat()))
  {
    switch(Get_Tag())
    {
      case (byte)25:
          switch(Get_Wochentag())
          {
            case SONNTAG:
              ret = OberhalbDiagonale();
              break;
          
            default:
              ret = UnterhalbDiagonale();
              break;
          }
          break;

      case (byte)26:
          switch(Get_Wochentag())
          {
            case SONNTAG:
            case MONTAG:
              ret = OberhalbDiagonale();
              break;
          
            default:
              ret = UnterhalbDiagonale();
              break;
          }
          break;

        case (byte)27:
          switch(Get_Wochentag())
          {
            case SONNTAG:
            case MONTAG:
            case DIENSTAG:
              ret = OberhalbDiagonale();
              break;
          
            default:
              ret = UnterhalbDiagonale();
              break;
          }
          break;

        case (byte)28:
          switch(Get_Wochentag())
          {
            case SONNTAG:
            case MONTAG:
            case DIENSTAG:
            case MITTWOCH:
              ret = OberhalbDiagonale();
              break;
          
            default:
              ret = UnterhalbDiagonale();
              break;
          }
          break;

        case (byte)29:
          switch(Get_Wochentag())
          {
            case SONNTAG:
            case MONTAG:
            case DIENSTAG:
            case MITTWOCH:
            case DONNERSTAG:
              ret = OberhalbDiagonale();
              break;
          
            default:
              ret = UnterhalbDiagonale();
              break;
          }
          break;

        case (byte)30:
          switch(Get_Wochentag())
          {
            case SONNTAG:
            case MONTAG:
            case DIENSTAG:
            case MITTWOCH:
            case DONNERSTAG:
            case FREITAG:
              ret = OberhalbDiagonale();
              break;
          
            default:
              ret = UnterhalbDiagonale();
              break;
          }
          break;

        case (byte)31:
          switch(Get_Wochentag())
          {
          
            default:
              ret = OberhalbDiagonale();
              break;
          }
          break;

        default:
        {
          if (MAERZ == Get_Monat())
          {
            ret = WinterZeit;
          }
          if (OKTOBER == Get_Monat())
          {
            ret = SommerZeit;
          }
        }
    }
  }
  return(ret); 
}

static JahresZeiten OberhalbDiagonale(void)
{
  JahresZeiten ret = SommerZeit;
  
  if(MAERZ == Get_Monat())
  {
    ret = SommerZeit;
  }
  if(OKTOBER == Get_Monat())
  {
    ret = WinterZeit;
  }
  return(ret);
}

static JahresZeiten UnterhalbDiagonale(void)
{
  JahresZeiten ret = SommerZeit;
  
  if(MAERZ == Get_Monat())
  {
    ret = WinterZeit;
  }
  if(OKTOBER == Get_Monat())
  {
    ret = SommerZeit;
  }
  return(ret);
}


